# name: systemd docker container
# desc: definition manifest for docker container defined via unit file
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

define systemd_docker::container (
	# P U P P E T   R E S O U R C E   M A N A G E M E N T
	Optional[Variant[Enum['absent', 'present'], Default]]
	                         $ensure                        = default,
	# D O C K E R   R U N   P A R A M E T E R S
	Optional[Pattern[/^[^\n]+$/]]
	                         $docker_run_executable         = $systemd_docker::docker_run_executable,
	Optional[Pattern[/^\w+$/]]
	                         $hostname                      = undef,
	# domainname has no equivalent on `docker run` CLI
	#Optional[Pattern[/^\w+$/]]
	#                         $domainname                    = undef,
	Optional[Struct[{
			user                          => Variant[Pattern[/^\w+$/], Integer[0]],
			group                         => Optional[Variant[Pattern[/^\w+$/], Integer[0]]],
		}]] $user = undef,
	Optional[Boolean]        $attach_stdin                  = undef,
	Optional[Boolean]        $attach_stdout                 = undef,
	Optional[Boolean]        $attach_stderr                 = undef,
	Optional[Boolean]        $tty                           = undef,
	Optional[Boolean]        $open_stdin                    = undef,
	Optional[Boolean]        $stdin_once                    = undef,
	Optional[Array[String]]  $env                           = undef, # TODO: data type
	Optional[Hash[String[1], String, 1]]
	                         $labels                        = undef,
	Optional[Array[String]]  $cmd                           = undef, # TODO: data type
	Optional[String]         $entrypoint                    = undef, # TODO: data type
	Struct[{
			# name can contain a colon ':' -- it separates the port number
			name                          => Pattern[/^[^[:blank:]]+$/],
			tag                           => Optional[Pattern[/^[^[:blank:]:]+$/]],
		}] $image = undef,
	Optional[Array[Struct[{
				volume_name                   => Pattern[/^[a-z0-9_.-]+$/],
				container_dir                 => Pattern[/^\/\S+\/$/, /^\/$/],
				options                       => Optional[Array[Variant[
						Enum['rw', 'ro'],
						Enum['z', 'Z'],
						Pattern[/^r?shared$/, /^r?slave$/, /^r?private$/],
						Enum['nocopy']
					], 1]],
			}], 1]] $volumes = undef,
	Optional[Pattern[/^\/\S+\/$/, /^\/$/]]
	                         $working_dir                   = undef,
	Optional[Boolean]        $network_disabled              = undef,
	Optional[Array[Integer[0x00, 0xFF], 6, 6]]
	                         $mac_address                   = undef,
	Optional[Array[Struct[{
				port                          => Integer[0, 65535],
				transport_protocol            => Enum['tcp', 'udp'],
			}]]] $exposed_ports = undef,
	Optional[String]         $stop_signal                   = undef,
	# H O S T _ C O N F I G
	Optional[Array[Variant[
			# file
			Struct[{
				host_path                     => Pattern[/^\/\S+[^[:blank:]\/]$/],
				container_path                => Pattern[/^\/\S+[^[:blank:]\/]$/],
				read_only                     => Optional[Boolean],
			}],
			# directory
			Struct[{
				host_path                     => Pattern[/^\/\S+\/$/, /^\/$/],
				container_path                => Pattern[/^\/\S+\/$/, /^\/$/],
				read_only                     => Optional[Boolean],
			}],
			# volume to directory
			Struct[{
				volume_name                   => Pattern[/^[a-z0-9_.-]+$/],
				container_path                => Pattern[/^\/\S+\/$/, /^\/$/],
				read_only                     => Optional[Boolean],
			}]
		], 1]] $binds = undef,
	Optional[Array[Struct[{
			container_name                => String[1],
			alias                         => String[1],
		}], 1]] $links = undef,
	Optional[Integer[0]]     $memory                        = undef,
	Optional[Integer[-1]]    $memory_swap                   = undef,
	Optional[Integer[0]]     $memory_reservation            = undef,
	Optional[Integer[0]]     $kernel_memory                 = undef,
	# CpuPercent is on M$ only
	Optional[Integer]        $cpu_shares                    = undef,
	Optional[Integer[0]]     $cpu_period                    = undef,
	Optional[Integer[0]]     $cpu_quota                     = undef,
	Optional[Array[Integer[0], 1]]
	                         $cpuset_cpus                   = undef,
	Optional[Array[Integer[0], 1]]
	                         $cpuset_mems                   = undef,
	Optional[Integer[0]]     $io_maximum_bandwidth          = undef,
	Optional[Integer[0]]     $io_maximum_iops               = undef,
	Optional[Integer[10, 1000]]
	                         $blkio_weight                  = undef,
	Optional[Array[Struct[{
			device_path                   => Pattern[/^\/\S+[^[:blank:]\/]$/],
			weight                        => Integer[10, 1000],
		}], 1]] $blkio_weight_device = undef,
	Optional[Array[Struct[{
			device_path                   => Pattern[/^\/\S+[^[:blank:]\/]$/],
			rate                          => Integer[0],
		}], 1]] $blkio_weight_read_bps = undef,
	Optional[Array[Struct[{
			device_path                   => Pattern[/^\/\S+[^[:blank:]\/]$/],
			rate                          => Integer[0],
		}], 1]] $blkio_weight_write_bps = undef,
	Optional[Array[Struct[{
			device_path                   => Pattern[/^\/\S+[^[:blank:]\/]$/],
			rate                          => Integer[0],
		}], 1]] $blkio_weight_read_iops = undef,
	Optional[Array[Struct[{
			device_path                   => Pattern[/^\/\S+[^[:blank:]\/]$/],
			rate                          => Integer[0],
		}], 1]] $blkio_weight_write_iops = undef,
	Optional[Integer[0, 100]]
	                         $memory_swappiness             = undef,
	Optional[Boolean]        $oom_kill_disable              = undef,
	Optional[Integer[-1000, 1000]]
	                         $oom_score_adj                 = undef,
	Optional[String]         $pid_mode                      = undef, # TODO: data type
	Optional[Integer[-1]]    $pids_limit                    = undef,
	Optional[Array[Struct[{
			port                          => Integer[0, 65535],
			protocol                      => Enum['tcp', 'udp'],
			host_port                     => Integer[0, 65535],
		}], 1]] $port_bindings = undef,
	Optional[Boolean]        $publish_all_ports             = undef,
	Optional[Boolean]        $privileged                    = undef,
	Optional[Boolean]        $readonly_rootfs               = undef,
	Optional[Array[Variant[
			Array[Integer[0, 255], 4, 4],
			Array[Integer[0x0000, 0xFFFF], 8, 8]
		], 1]] $dns = undef,
	Optional[Struct[{
			debug                         => Optional[Boolean],
			ndots                         => Optional[Integer[0, 15]],
			timeout                       => Optional[Integer[0, 30]],
			attempts                      => Optional[Integer[0, 5]],
			rotate                        => Optional[Boolean],
			no_check_names                => Optional[Boolean],
			inet6                         => Optional[Boolean],
			ip6_bytestring                => Optional[Boolean],
			ip6_dotint                    => Optional[Boolean],
			edns0                         => Optional[Boolean],
			single_request                => Optional[Boolean],
			single_request_reopen         => Optional[Boolean],
		}]] $dns_options = undef,
	Optional[Array[String[1], 1]]
	                         $dns_search                    = undef,
	Optional[Array[Struct[{
			hostname                      => String[1],
			ip                            => Variant[
					Array[Integer[0, 255], 4, 4],
					Array[Integer[0x0000, 0xFFFF], 8, 8]
				],
		}], 1]] $extra_hosts = undef,
	Optional[Array[Struct[{
			container_name                => String[1],
			read                          => Optional[Enum['ro', 'rw']],
		}], 1]] $volumes_from = undef,
	Optional[Array[String[1], 1]]
	                         $cap_add                       = undef, # TODO: restrict
	Optional[Array[String[1], 1]]
	                         $cap_drop                      = undef, # TODO: restrict
	Optional[Array[String[1], 1]]
	                         $group_add                     = undef,
	Optional[Variant[
			Struct[{
				name                          => Enum['always', 'unless-stopped', 'no'],
			}],
			Struct[{
				name                          => Enum['on-failure'],
				maximum_retry_count           => Integer[0],
			}]
		]] $restart_policy = undef,
	Optional[Boolean]        $auto_remove                   = undef,
	Optional[Enum['host']]   $userns_mode                   = undef,
	Optional[String[1]]      $network_mode                  = undef,
	Optional[Array[Struct[{
			path_on_host                  => String[1],
			path_in_container             => String[1],
			cgroup_permissions            => Optional[Pattern[/^rw?m?$/, /^r?wm?$/, /^r?w?m$/]],
		}], 1]] $devices = undef,
	Optional[Array[Struct[{
			name                          => String[1],
			soft_limit                    => Integer[0],
			hard_limit                    => Integer[0],
		}], 1]] $ulimits = undef,
	Optional[Hash[String[1], String[1], 1]]
	                         $sysctls                       = undef,
	Optional[Array[String[1], 1]]
	                         $security_opt                  = undef,
	Optional[Undef]          $storage_opt                   = undef, # TODO
	Optional[Undef]          $log_config                    = undef, # TODO
	Optional[String]         $cgroup_parent                 = undef, # TODO
	Optional[String[1]]      $volume_driver                 = undef,
	Optional[Integer[1]]     $shm_size                      = undef,
	# U N I T   P A R A M E T E R S
	# <prefix_name>.service file (and dependent)
	Variant[Pattern[/^\/\S*[^[:blank:]\/]$/], Default]
	                         $path                          = default,
	Optional[Variant[String[1], Default]]
	                         $mode                          = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $owner                         = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $group                         = default,
	Variant[String[1], Default]
	                         $content_template              = default,
	Optional[Variant[String[1], Default]]
	                         $validate_cmd                  = undef,
	Optional[String[1]]
	                         $validate_replacement          = undef,
	Optional[Enum[
			'md5',
			'md5lite',
			'sha256',
			'sha256lite',
			'mtime',
			'ctime',
			'none'
		]] $checksum = undef,
	Optional[Boolean]        $show_diff                     = undef,
	Optional[Variant[Boolean, String]]
	                         $backup                        = undef,
	String                   $blame                         = $systemd::params::systemd_unit_default_owner,
	# <unit_name>.wants/ directory
	Boolean                  $manage_wants_dir              = false,
	Optional[Variant[String[1], Default]]
	                         $wants_dir_mode                = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $wants_dir_owner               = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $wants_dir_group               = default,
	Boolean                  $wants_dir_complete_control    = false,
	# <unit_name>.requires/ directory
	Boolean                  $manage_requires_dir           = false,
	Optional[Variant[String[1], Default]]
	                         $requires_dir_mode             = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $requires_dir_owner            = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $requires_dir_group            = default,
	Boolean                  $requires_dir_complete_control = false,
	# <unit_name>.d/ directory
	Boolean                  $manage_conf_dir               = false,
	Optional[Variant[String[1], Default]]
	                         $conf_dir_mode                 = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $conf_dir_owner                = default,
	Optional[Variant[Integer[0], String[1], Default]]
	                         $conf_dir_group                = default,
	Boolean                  $conf_dir_complete_control     = false,
	# U N I T
	# unit metadata
	Optional[String]         $unit_description              = undef,
	Optional[Array[Pattern[/^(https?:\/\/|file:|info:|man:)\S+$/, /^$/], 1]]
	                         $unit_documentation            = undef,
	# unit dependencies
	Variant[Array[Pattern[/^\S+$/], 1], Default, Undef]
	                         $unit_requires                 = default,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requires_overridable     = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requisite                = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_requisite_overridable    = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_wants                    = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_binds_to                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_part_of                  = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_conflicts                = undef,
	# unit ordering
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_before                   = undef,
	Variant[Array[Pattern[/^\S+$/], 1], Default, Undef]
	                         $unit_after                    = default,
	# unit behavior
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_on_failure               = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_propagates_reload_to     = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_reload_propagated_from   = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $unit_joins_namespace_of       = undef,
	# absolute path (starts with slash)
	Optional[Array[Pattern[/^\/\S+$/], 1]]
	                         $unit_requires_mount_for       = undef,
	Optional[Enum[
			'fail',
			'replace',
			'replace-irreversibly',
			'isolate',
			'flush',
			'ignore-dependencies',
			'ignore-requirements'
		]] $unit_on_failure_job_mode = undef,
	Optional[Boolean]        $unit_ignore_on_isolate        = undef,
	Optional[Boolean]        $unit_ignore_on_snapshot       = undef,
	Optional[Boolean]        $unit_stop_when_unneeded       = undef,
	Optional[Boolean]        $unit_refuse_manual_start      = undef,
	Optional[Boolean]        $unit_refuse_manual_stop       = undef,
	Optional[Boolean]        $unit_allow_isolate            = undef,
	Optional[Boolean]        $unit_default_dependencies     = undef,
	Optional[Integer[0]]     $unit_job_timeout_sec          = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $unit_job_timeout_action = undef,
	Optional[String[1]]      $unit_job_timeout_reboot_arg   = undef,
	# unit conditions
	Optional[Pattern[
			/^\|?!?x86(-64)?$/,
			/^\|?!?ppc(-le)?$/,
			/^\|?!?ppc64(-le)?$/,
			/^\|?!?ia64$/,
			/^\|?!?parisc(64)?$/,
			/^\|?!?s390x?$/,
			/^\|?!?sparc(64)?$/,
			/^\|?!?mips(-le)?$/,
			/^\|?!?mips64(-le)?$/,
			/^\|?!?alpha$/,
			/^\|?!?arm(-be)?$/,
			/^\|?!?arm64(-be)?$/,
			/^\|?!?sh(64)?$/,
			/^\|?!?m86k$/,
			/^\|?!?tilegx$/,
			/^\|?!?cris$/
		]] $unit_condition_architecture = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $unit_condition_virtualization = undef,
	Optional[Pattern[/^\|?!?\S+$/]]
	                         $unit_condition_host           = undef,
	Optional[Pattern[/^\|?\S+$/]]
	                         $unit_condition_kernel_cmdline = undef,
	Optional[Pattern[/^\|?!?(selinux|apparmor|ima|smack|audit)$/]]
	                         $unit_condition_security       = undef,
	Optional[Pattern[
			/^\|?!?CAP_AUDIT_(CONTROL|READ|WRITE)$/,
			/^\|?!?CAP_BLOCK_SUSPEND$/,
			/^\|?!?CAP_CHOWN$/,
			/^\|?!?CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
			/^\|?!?CAP_FOWNER$/,
			/^\|?!?CAP_FSETID$/,
			/^\|?!?CAP_IPC_(LOCK|OWNER)$/,
			/^\|?!?CAP_KILL$/,
			/^\|?!?CAP_LEASE$/,
			/^\|?!?CAP_LINUX_IMMUTABLE$/,
			/^\|?!?CAP_MAC_(ADMIN|OVERRIDE)$/,
			/^\|?!?CAP_MKNOD$/,
			/^\|?!?CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
			/^\|?!?CAP_SETGID$/,
			/^\|?!?CAP_SETFCAP$/,
			/^\|?!?CAP_SETUID$/,
			/^\|?!?CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
			/^\|?!?CAP_SYSLOG$/,
			/^\|?!?CAP_WAKE_ALARM$/
		]] $unit_condition_capability = undef,
	Optional[Boolean]        $unit_condition_ac_power       = undef,
	Optional[Pattern[/^\|?!?\/(etc|var)$/]]
	                         $unit_condition_needs_update   = undef,
	Optional[Boolean]        $unit_condition_first_boot     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_exists    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_exists_gl = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_dir    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_syml   = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_mntpt  = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_path_is_rw     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_dir_not_empty  = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_file_not_empty = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_condition_file_is_exec   = undef,
	# unit assertion
	Optional[Pattern[
			/^\|?!?x86(-64)?$/,
			/^\|?!?ppc(-le)?$/,
			/^\|?!?ppc64(-le)?$/,
			/^\|?!?ia64$/,
			/^\|?!?parisc(64)?$/,
			/^\|?!?s390x?$/,
			/^\|?!?sparc(64)?$/,
			/^\|?!?mips(-le)?$/,
			/^\|?!?mips64(-le)?$/,
			/^\|?!?alpha$/,
			/^\|?!?arm(-be)?$/,
			/^\|?!?arm64(-be)?$/,
			/^\|?!?sh(64)?$/,
			/^\|?!?m86k$/,
			/^\|?!?tilegx$/,
			/^\|?!?cris$/
		]] $unit_assert_architecture = undef,
	Optional[Variant[
			Boolean,
			Pattern[/^\|?!?(vm|container)$/],
			Pattern[/^\|?!?(qemu|kvm|zvm|vmware|microsoft|oracle|xen|bochs|uml)$/],
			Pattern[/^\|?!?(openvz|lxc|lxc-libvirt|systemd-nspawn|docker)$/]
		]] $unit_assert_virtualization = undef,
	Optional[Pattern[/^\|?!?\S+$/]]
	                         $unit_assert_host              = undef,
	Optional[Pattern[/^\|?\S+$/]]
	                         $unit_assert_kernel_cmdline    = undef,
	Optional[Pattern[/^\|?!?(selinux|apparmor|ima|smack|audit)$/]]
	                         $unit_assert_security          = undef,
	Optional[Pattern[
			/^\|?!?CAP_AUDIT_(CONTROL|READ|WRITE)$/,
			/^\|?!?CAP_BLOCK_SUSPEND$/,
			/^\|?!?CAP_CHOWN$/,
			/^\|?!?CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
			/^\|?!?CAP_FOWNER$/,
			/^\|?!?CAP_FSETID$/,
			/^\|?!?CAP_IPC_(LOCK|OWNER)$/,
			/^\|?!?CAP_KILL$/,
			/^\|?!?CAP_LEASE$/,
			/^\|?!?CAP_LINUX_IMMUTABLE$/,
			/^\|?!?CAP_MAC_(ADMIN|OVERRIDE)$/,
			/^\|?!?CAP_MKNOD$/,
			/^\|?!?CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
			/^\|?!?CAP_SETGID$/,
			/^\|?!?CAP_SETFCAP$/,
			/^\|?!?CAP_SETUID$/,
			/^\|?!?CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
			/^\|?!?CAP_SYSLOG$/,
			/^\|?!?CAP_WAKE_ALARM$/
		]] $unit_assert_capability = undef,
	Optional[Boolean]        $unit_assert_ac_power          = undef,
	Optional[Pattern[/^\|?!?\/(etc|var)$/]]
	                         $unit_assert_needs_update      = undef,
	Optional[Boolean]        $unit_assert_first_boot        = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_exists       = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_exists_gl    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_dir       = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_syml      = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_mntpt     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_path_is_rw        = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_dir_not_empty     = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_file_not_empty    = undef,
	Optional[Pattern[/^\|?!?\/\S+$/]]
	                         $unit_assert_file_is_exec      = undef,
	Optional[String[1]]      $unit_source_path              = undef,
	# I N S T A L L
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_alias                 = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_wanted_by             = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_required_by           = undef,
	Optional[Array[Pattern[/^\S+$/], 1]]
	                         $install_also                  = undef,
	Optional[Pattern[/^\S+$/]]
	                         $install_default_instance      = undef,
	# custom sections
	Optional[Hash[String[1], Hash[String[1], Any, 1], 1]]
	                         $custom_options                = undef,
	# S E R V I C E
	# native [Service] section settings
	Optional[Enum['simple', 'forking', 'oneshot', 'dbus', 'notify', 'idle']]
	                         $service_type                  = undef,
	Optional[Boolean]        $service_remain_after_exit     = undef,
	Optional[Boolean]        $service_guess_main_pid        = undef,
	Optional[Pattern[/^\/\S+$/]]
	                         $service_pid_file              = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_bus_name              = undef,
	Optional[Array[Struct[{
			bus_name                      => Pattern[/^\S+$/],
			permission                    => Enum['see', 'talk', 'own'],
		}], 1]] $service_bus_policy = undef,
	Optional[Variant[Array[String, 1], Default]]
	                         $service_exec_start_pre        = default,
	Optional[Array[String, 1]]
	                         $service_exec_start            = undef,
	Optional[Array[String, 1]]
	                         $service_exec_start_post       = undef,
	Optional[Array[String, 1]]
	                         $service_exec_reload           = undef,
	Optional[Array[String, 1]]
	                         $service_exec_stop             = undef,
	Optional[Array[String, 1]]
	                         $service_exec_stop_post        = undef,
	Optional[Integer[0]]     $service_restart_sec           = undef,
	Optional[Integer[0]]     $service_timeout_start_sec     = undef,
	Optional[Integer[0]]     $service_timeout_stop_sec      = undef,
	Optional[Integer[0]]     $service_timeout_sec           = undef,
	Optional[Integer[0]]     $service_watchdog_sec          = undef,
	Optional[Enum['no', 'on-success', 'on-failure', 'on-abnormal', 'on-watchdog', 'on-abort', 'always']]
	                         $service_restart               = undef,
	Optional[Variant[
			Integer[0, 255],
			Enum[
				'SIGHUP',
				'SIGINT',
				'SIGKILL',
				'SIGPIPE',
				'SIGALRM',
				'SIGTERM'
			]
		]] $service_success_exit_status = undef,
	Optional[Variant[
			Integer[0, 255],
			Enum[
				'SIGHUP',
				'SIGINT',
				'SIGKILL',
				'SIGPIPE',
				'SIGALRM',
				'SIGTERM'
			]
		]] $service_restart_prevnt_exit_s = undef,
	Optional[Variant[
			Integer[0, 255],
			Enum[
				'SIGHUP',
				'SIGINT',
				'SIGKILL',
				'SIGPIPE',
				'SIGALRM',
				'SIGTERM'
			]
		]] $service_force_exit_status = undef,
	Optional[Boolean]        $service_root_dir_start_only   = undef,
	Optional[Boolean]        $service_non_blocking          = undef,
	Optional[Enum['none', 'main', 'all']]
	                         $service_notify_access         = undef,
	Optional[Array[Pattern[/^$/, /^\S+$/]]]
	                         $service_sockets               = undef,
	Optional[Integer[0]]     $service_start_limit_interval  = undef,
	Optional[Integer[0]]     $service_start_limit_burst     = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $service_start_limit_action = undef,
	Optional[Enum[
			'none',
			'reboot',
			'reboot-force',
			'reboot-immediate',
			'poweroff',
			'poweroff-force',
			'poweroff-immediate'
		]] $service_failure_action = undef,
	Optional[String]         $service_reboot_argument       = undef,
	Optional[Integer[0]]     $service_fd_store_max          = undef,
	# systemd.exec:
	Optional[Pattern[/^\/$/, /^\/\S+\/$/]]
	                         $service_working_directory     = undef,
	Optional[Pattern[/^\/$/, /^\/\S+\/$/]]
	                         $service_root_directory        = undef,
	Optional[Variant[Integer[0], Pattern[/^\S+$/]]]
	                         $service_user                  = undef,
	Optional[Variant[Integer[0], Pattern[/^\S+$/]]]
	                         $service_group                 = undef,
	Optional[Array[Variant[Integer[0], Pattern[/^$/, /^\S+$/]], 1]]
	                         $service_supplementary_groups  = undef,
	Optional[Integer[-20, 19]]
	                         $service_nice                  = undef,
	Optional[Integer[-1000, 1000]]
	                         $service_oom_score_adjust      = undef,
	Optional[Variant[Integer[0, 3], Enum['none', 'realtime', 'best-effort', 'idle']]]
	                         $service_io_scheduling_class   = undef,
	Optional[Integer[0, 7]]  $service_io_scheduling_prio    = undef,
	Optional[Enum['other', 'batch', 'idle', 'fifo', 'rr']]
	                         $service_cpu_sched_policy      = undef,
	Optional[Integer[0, 99]] $service_cpu_sched_priority    = undef,
	Optional[Boolean]        $service_cpu_sched_rset_on_frk = undef,
	Optional[Array[Variant[Integer[0], Enum['']], 1]]
	                         $service_cpu_affinity          = undef,
	Optional[Pattern[/^[0-7]{4}$/]]
	                         $service_umask                 = undef,
	Optional[Array[Pattern[
			/^$/,
			/^"\S+=.*"$/,
			/^\S+=\S+$/
		], 1]] $service_environment = undef,
	Optional[Array[Pattern[
			/^-?\/\S+$/,
			/^$/
		], 1]] $service_environment_file = undef,
	Optional[Enum['null', 'tty', 'tty-force', 'tty-fail', 'socket']]
	                         $service_standard_input        = undef,
	Optional[Enum[
			'inherit',
			'tty',
			'journal',
			'syslog',
			'kmsg',
			'journal+console',
			'syslog+console',
			'kmsg+console',
			'socket',
			'null'
		]] $service_standard_output = undef,
	Optional[Enum[
			'inherit',
			'tty',
			'journal',
			'syslog',
			'kmsg',
			'journal+console',
			'syslog+console',
			'kmsg+console',
			'socket',
			'null'
		]] $service_standard_error = undef,
	Optional[String]         $service_tty_path              = undef,
	Optional[Boolean]        $service_tty_reset             = undef,
	Optional[Boolean]        $service_ttyv_hangup           = undef,
	Optional[Boolean]        $service_ttyvt_disallocate     = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_syslog_identifier     = undef,
	Optional[Pattern[
			/^kern$/,
			/^user$/,
			/^mail$/,
			/^daemon$/,
			/^auth$/,
			/^syslog$/,
			/^lpr$/,
			/^news$/,
			/^uucp$/,
			/^cron$/,
			/^authpriv$/,
			/^ftp$/,
			/^local[0-7]$/
		]] $service_syslog_facility = undef,
	Optional[Enum[
			'emerg',
			'alert',
			'crit',
			'err',
			'warning',
			'notice',
			'info',
			'debug'
		]] $service_syslog_level = undef,
	Optional[Boolean]        $service_syslog_level_prefix   = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_timer_slack_nsec      = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_cpu             = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_fsize           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_data            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_stack           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_core            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_rss             = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_nofile          = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_as              = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_nproc           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_memlock         = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_locks           = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_sigpending      = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_msgqueue        = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_nice            = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_rtprio          = undef,
	Optional[Variant[Integer[0], Enum['infinity']]]
	                         $service_limit_rttime          = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_pam_name              = undef,
	Optional[Array[Struct[{
			invert_selection              => Boolean,
			capabilities                  => Array[Pattern[
					/^CAP_AUDIT_(CONTROL|READ|WRITE)$/,
					/^CAP_BLOCK_SUSPEND$/,
					/^CAP_CHOWN$/,
					/^CAP_DAC_(OVERRIDE|READ_SEARCH)$/,
					/^CAP_FOWNER$/,
					/^CAP_FSETID$/,
					/^CAP_IPC_(LOCK|OWNER)$/,
					/^CAP_KILL$/,
					/^CAP_LEASE$/,
					/^CAP_LINUX_IMMUTABLE$/,
					/^CAP_MAC_(ADMIN|OVERRIDE)$/,
					/^CAP_MKNOD$/,
					/^CAP_NET_(ADMIN|BIND_SERVICE|BROADCAST|RAW)$/,
					/^CAP_SETGID$/,
					/^CAP_SETFCAP$/,
					/^CAP_SETUID$/,
					/^CAP_SYS_(ADMIN|BOOT|CHROOT|MODULE|NICE|PACCT|PTRACE|RAWIO|RESOURCE|TIME|TTY_CONFIG)$/,
					/^CAP_SYSLOG$/,
					/^CAP_WAKE_ALARM$/
				], 0],
		}], 1]] $service_capability_boundn_set = undef,
	Optional[Array[Pattern[
			/^keep-caps(-locked)?$/,
			/^no-setuid-fixup(-locked)?$/,
			/^noroot(-locked)?$/,
			/^$/
		], 1]] $service_secure_bits = undef,
	Optional[Array[Struct[{
			capabilities                  => Variant[
					Array[Enum['all'], 1, 1],
					Array[Variant[
							Pattern[
								/^cap_audit_(control|read|write)$/,
								/^cap_block_suspend$/,
								/^cap_chown$/,
								/^cap_dac_(override|read_search)$/,
								/^cap_fowner$/,
								/^cap_fsetid$/,
								/^cap_ipc_(lock|owner)$/,
								/^cap_kill$/,
								/^cap_lease$/,
								/^cap_linux_immutable$/,
								/^cap_mac_(admin|override)$/,
								/^cap_mknod$/,
								/^cap_net_(admin|bind_service|broadcast|raw)$/,
								/^cap_setgid$/,
								/^cap_setfcap$/,
								/^cap_setuid$/,
								/^cap_sys_(admin|boot|chroot|module|nice|pacct|ptrace|rawio|resource|time|tty_config)$/,
								/^cap_syslog$/,
								/^cap_wake_alarm$/
							],
							Integer[0]
						], 1]
				],
			action_list                   => Array[Variant[
					Struct[{
						operator                      => Enum['='],
						flag                          => Optional[Pattern[
								/^ei?p?$/,
								/^e?ip?$/,
								/^e?i?p$/
							]],
					}],
					Struct[{
						operator                      => Enum['+', '-'],
						flag                          => Pattern[/^ei?p?$/, /^e?ip?$/, /^e?i?p$/],
					}]
				], 1]
		}], 1]] $service_capabilities = undef,
	Optional[Array[Pattern[/^$/, /^\/\S+\/$/], 1]]
	                         $service_read_write_dirs       = undef,
	Optional[Array[Pattern[/^$/, /^-?\/\S+\/$/], 1]]
	                         $service_read_only_dirs        = undef,
	Optional[Array[Pattern[/^$/, /^-?\/\S+\/$/], 1]]
	                         $service_inaccessible_dirs     = undef,
	Optional[Boolean]        $service_private_tmp           = undef,
	Optional[Boolean]        $service_private_devices       = undef,
	Optional[Boolean]        $service_private_network       = undef,
	Optional[Variant[Boolean, Enum['full']]]
	                         $service_protect_system        = undef,
	Optional[Variant[Boolean, Enum['read-only']]]
	                         $service_protect_home          = undef,
	Optional[Enum['shared', 'slave', 'private']]
	                         $service_mount_flags           = undef,
	Optional[Pattern[/^\S+$/]]
	                         $service_utmp_identifier       = undef,
	Optional[Enum['init', 'login', 'user']]
	                         $service_utmp_mode             = undef,
	Optional[Pattern[/^-?\S+$/]]
	                         $service_se_linux_context      = undef,
	Optional[Pattern[/^-?\S+$/]]
	                         $service_app_armor_profile     = undef,
	Optional[Pattern[/^-?\S+$/, /^$/]]
	                         $service_smack_process_label   = undef,
	Optional[Boolean]        $service_ignore_sigpipe        = undef,
	Optional[Boolean]        $service_no_new_privileges     = undef,
	Optional[Array[Struct[{
			invert_selection              => Boolean,
			system_calls                  => Variant[
					Array[Enum[''], 1, 1],
					Array[Pattern[/^\S+$/], 1]
				],
		}], 1]] $service_system_call_filter = undef,
	Optional[Pattern[/^$/, /^\S+$/]]
	                         $service_system_call_error_no  = undef,
	Optional[Array[Enum['x86', 'x86-64', 'x32', 'arm', 'native'], 1]]
	                         $service_system_call_archs     = undef,
	Optional[Struct[{
			invert_selection              => Boolean,
			address_families              => Variant[
					Array[Enum[
						'AF_APPLETALK',
						'AF_ASH',
						'AF_ATMPVC',
						'AF_ATMSVC',
						'AF_AX25',
						'AF_BLUETOOTH',
						'AF_BRIDGE',
						'AF_DECnet',
						'AF_ECONET',
						'AF_INET',
						'AF_INET6',
						'AF_IPX',
						'AF_IRDA',
						'AF_KEY',
						'AF_LLC',
						'AF_LOCAL',
						'AF_MAX',
						'AF_NETBEUI',
						'AF_NETLINK',
						'AF_NETROM',
						'AF_PACKET',
						'AF_PPPOX',
						'AF_ROSE',
						'AF_ROUTE',
						'AF_SECURITY',
						'AF_SNA',
						'AF_TIPC',
						'AF_UNSPEC',
						'AF_WANPIPE',
						'AF_X25',
						'AF_UNIX'
					], 1, 31],
					Array[Enum[''], 1, 1]
				],
		}]] $service_restrict_address_fam  = undef,
	Optional[Pattern[/^x86(-64)?$/]]
	                         $service_personality           = undef,
	Optional[Array[Pattern[/^[^[:blank:]\/]+$/], 1]]
	                         $service_runtime_dir           = undef,
	Optional[Pattern[/^[0-7]{4}$/]]
	                         $service_runtime_dir_mode      = undef,
	# systemd.kill:
	Optional[Enum['control-group', 'process', 'mixed', 'none']]
	                         $service_kill_mode             = undef,
	Optional[Enum[
			'SIGHUP',
			'SIGINT',
			'SIGPIPE',
			'SIGALRM',
			'SIGTERM',
			'SIGUSR1',
			'SIGUSR2',
			'SIGKILL'
		]] $service_kill_signal = undef,
	Optional[Boolean]        $service_send_sighup           = undef,
	Optional[Boolean]        $service_send_sigkill          = undef,
	# settings imported from systemd.resource-control
	Optional[Boolean]        $service_cpu_accounting        = undef,
	Optional[Integer]        $service_cpu_shares            = undef,
	Optional[Integer]        $service_startup_cpu_shares    = undef,
	Optional[Integer[0]]     $service_cpu_quota             = undef,
	Optional[Boolean]        $service_memory_accounting     = undef,
	Optional[Integer[0]]     $service_memory_limit          = undef,
	Optional[Boolean]        $service_blk_io_accounting     = undef,
	Optional[Integer[10, 1000]]
	                         $service_blk_io_weight         = undef,
	Optional[Integer[10, 1000]]
	                         $service_startup_blk_io_weight = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			weight                        => Integer[10, 1000],
		}], 1]] $service_blk_io_device_weight = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			bandwidth_bytes_per_second    => Integer[0],
		}], 1]] $service_blk_io_read_bw = undef,
	Optional[Array[Struct[{
			device                        => String[1],
			bandwidth_bytes_per_second    => Integer[0],
		}], 1]] $service_blk_io_write_bw = undef,
	Optional[Struct[{
			device                        => String[1],
			access                        => Pattern[/^rw?m?$/, /^r?wm?$/, /^r?w?m$/],
		}]] $service_device_allow = undef,
	Optional[Enum['auto', 'closed', 'strict']]
	                         $service_device_policy         = undef,
	Optional[String[1]]      $service_slice                 = undef,
	Optional[Boolean]        $service_delegate              = undef,
	# P U P P E T   E N H A N C E M E N T S
	Boolean                  $manage_service_resource       = false,
	Variant[Default, Enum['stopped', 'running']]
	                         $ensure_service                = default,
	Optional[Variant[Boolean, Enum['manual', 'mask']]]
	                         $enable_service                = undef,
	Optional[Boolean]        $hasrestart                    = undef,
	Optional[Boolean]        $hasstatus                     = undef,
	Boolean                  $explicit_provider             = false,
	Boolean                  $refresh_service_unit_update   = false,
) {
	#                                                                       #
	# I M P L E M E N T A T I O N                                           #
	#                                                                       #
	
	# init
	
	# me variable for proper error messages
	$_me = sprintf('%s', Systemd_docker::Container[$title])
	
	# input validation
	#if ($volumes != undef) {
	#	$volumes.each |Integer $i, Hash $v| {
	#		$o = $v['options']
	#		if ($o != undef) {
	#			if (('rw' in $o) and ('ro' in $o)) {
	#				fail("${_me}: volume \$volumes[${i}]['options'] contains 'rw' _and_ 'ro'")
	#			}
	#			if (('z' in $o) and ('Z' in $o)) {
	#				fail("${_me}: volume \$volumes[${i}]['options'] contains 'z' _and_ 'Z'")
	#			}
	#			if (((/^r?shared$/ in $o) and (/^r?slave$/ in $o)) or
	#				((/^r?shared$/ in $o) and (/^r?private$/ in $o)) or
	#				((/^r?slave$/ in $o) and (/^r?private$/ in $o))) {
	#				fail("${_me}: at least two of /^r?shared$/, /^r?slave$/, /^r?private$/ are in \$volumes[${i}]['options']")
	#			}
	#		}
	#	}
	#}
	if ($host_config != undef) {
		if ($host_config['memory_swap'] != undef) {
			if ($host_config['memory'] == undef) {
				warning("${_me}: \$host_config['memory_swap'] has to be used with \$host_config['memory']")
			} else {
				unless ($host_config['memory_swap'] > $host_config['memory']) {
					warning("${_me}: \$host_config['memory_swap'] has to be larger than \$host_config['memory']")
				}
			}
		}
	}
	
	if ($name == undef) {
		# by default we use title as container name: accept only valid container names
		if $title !~ /\/?[a-zA-Z0-9_-]+/ {
			fail("${_me}: \$title must match /?[a-zA-Z0-9_-]+ unless \$name specified")
		}
	}
	
	$_image                        = ($image['tag'] != undef) ? {
		true                          => "${image['name']}:${image['tag']}",
		default                       => $image['name'],
	}
	
	$_unit_requires                = ($unit_requires == default) ? {
		true                          => ['docker.service'],
		default                       => $unit_requires,
	}
	$_unit_after                   = ($unit_after == default) ? {
		true                          => ['docker.service'],
		default                       => $unit_after,
	}
	
	# reasonable approaches:
	# https://github.com/docker/docker/issues/6791
	
	# detach, if we do docker run in ExecStartPre
	$detach                        = true
	
	$_service_exec_start_pre       = ($service_exec_start_pre == default) ? {
		true                          => [
				# in case the container start was not supervised by systemd
				"-/usr/bin/docker stop -t 61 ${title}",
				# docker run refuses to start containers their name's already there
				"-/usr/bin/docker rm ${title}",
				"/usr/bin/docker pull ${_image}",
				template('systemd_docker/container_exec.erb')
			],
		default                       => $service_exec_start_pre,
	}
	
	# R E S O U R C E   D E C L A R A T I O N S
	systemd::service { $title:
		ensure                        => $ensure,
		path                          => $path,
		mode                          => $mode,
		owner                         => $owner,
		group                         => $group,
		content_template              => $content_template,
		validate_cmd                  => $validate_cmd,
		validate_replacement          => $validate_replacement,
		checksum                      => $checksum,
		show_diff                     => $show_diff,
		backup                        => $backup,
		blame                         => $blame,
		# <unit_name>.wants/ directory
		manage_wants_dir              => $manage_wants_dir,
		wants_dir_mode                => $wants_dir_mode,
		wants_dir_owner               => $wants_dir_owner,
		wants_dir_group               => $wants_dir_group,
		wants_dir_complete_control    => $wants_dir_complete_control,
		# <unit_name>.requires/ directory
		manage_requires_dir           => $manage_requires_dir,
		requires_dir_mode             => $requires_dir_mode,
		requires_dir_owner            => $requires_dir_owner,
		requires_dir_group            => $requires_dir_group,
		requires_dir_complete_control => $requires_dir_complete_control,
		manage_conf_dir               => $manage_conf_dir,
		conf_dir_mode                 => $conf_dir_mode,
		conf_dir_owner                => $conf_dir_owner,
		conf_dir_group                => $conf_dir_group,
		conf_dir_complete_control     => $conf_dir_complete_control,
		# U N I T
		# unit metadata
		unit_description              => $unit_description,
		unit_documentation            => $unit_documentation,
		# unit dependencies
		unit_requires                 => $_unit_requires,
		unit_requires_overridable     => $unit_requires_overridable,
		unit_requisite                => $unit_requisite,
		unit_requisite_overridable    => $unit_requisite_overridable,
		unit_wants                    => $unit_wants,
		unit_binds_to                 => $unit_binds_to,
		unit_part_of                  => $unit_part_of,
		unit_conflicts                => $unit_conflicts,
		# unit ordering
		unit_before                   => $unit_before,
		unit_after                    => $_unit_after,
		# unit behavior
		unit_on_failure               => $unit_on_failure,
		unit_propagates_reload_to     => $unit_propagates_reload_to,
		unit_reload_propagated_from   => $unit_reload_propagated_from,
		unit_joins_namespace_of       => $unit_joins_namespace_of,
		unit_requires_mount_for       => $unit_requires_mount_for,
		unit_on_failure_job_mode      => $unit_on_failure_job_mode,
		unit_ignore_on_isolate        => $unit_ignore_on_isolate,
		unit_ignore_on_snapshot       => $unit_ignore_on_snapshot,
		unit_stop_when_unneeded       => $unit_stop_when_unneeded,
		unit_refuse_manual_start      => $unit_refuse_manual_start,
		unit_refuse_manual_stop       => $unit_refuse_manual_stop,
		unit_allow_isolate            => $unit_allow_isolate,
		unit_default_dependencies     => $unit_default_dependencies,
		unit_job_timeout_sec          => $unit_job_timeout_sec,
		unit_job_timeout_action       => $unit_job_timeout_action,
		unit_job_timeout_reboot_arg   => $unit_job_timeout_reboot_arg,
		# unit conditions
		unit_condition_architecture   => $unit_condition_architecture,
		unit_condition_virtualization => $unit_condition_virtualization,
		unit_condition_host           => $unit_condition_host,
		unit_condition_kernel_cmdline => $unit_condition_kernel_cmdline,
		unit_condition_security       => $unit_condition_security,
		unit_condition_capability     => $unit_condition_capability,
		unit_condition_ac_power       => $unit_condition_ac_power,
		unit_condition_needs_update   => $unit_condition_needs_update,
		unit_condition_first_boot     => $unit_condition_first_boot,
		unit_condition_path_exists    => $unit_condition_path_exists,
		unit_condition_path_exists_gl => $unit_condition_path_exists_gl,
		unit_condition_path_is_dir    => $unit_condition_path_is_dir,
		unit_condition_path_is_syml   => $unit_condition_path_is_syml,
		unit_condition_path_is_mntpt  => $unit_condition_path_is_mntpt,
		unit_condition_path_is_rw     => $unit_condition_path_is_rw,
		unit_condition_dir_not_empty  => $unit_condition_dir_not_empty,
		unit_condition_file_not_empty => $unit_condition_file_not_empty,
		unit_condition_file_is_exec   => $unit_condition_file_is_exec,
		# unit assertion
		unit_assert_architecture      => $unit_assert_architecture,
		unit_assert_virtualization    => $unit_assert_virtualization,
		unit_assert_host              => $unit_assert_host,
		unit_assert_kernel_cmdline    => $unit_assert_kernel_cmdline,
		unit_assert_security          => $unit_assert_security,
		unit_assert_capability        => $unit_assert_capability,
		unit_assert_ac_power          => $unit_assert_ac_power,
		unit_assert_needs_update      => $unit_assert_needs_update,
		unit_assert_first_boot        => $unit_assert_first_boot,
		unit_assert_path_exists       => $unit_assert_path_exists,
		unit_assert_path_exists_gl    => $unit_assert_path_exists_gl,
		unit_assert_path_is_dir       => $unit_assert_path_is_dir,
		unit_assert_path_is_syml      => $unit_assert_path_is_syml,
		unit_assert_path_is_mntpt     => $unit_assert_path_is_mntpt,
		unit_assert_path_is_rw        => $unit_assert_path_is_rw,
		unit_assert_dir_not_empty     => $unit_assert_dir_not_empty,
		unit_assert_file_not_empty    => $unit_assert_file_not_empty,
		unit_assert_file_is_exec      => $unit_assert_file_is_exec,
		unit_source_path              => $unit_source_path,
		# I N S T A L L
		install_alias                 => $install_alias,
		install_wanted_by             => $install_wanted_by,
		install_required_by           => $install_required_by,
		install_also                  => $install_also,
		install_default_instance      => $install_default_instance,
		# C U S T O M   S E C T I O N S
		custom_options                => $custom_options,
		# S E R V I C E - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		service_type                  => $service_type,
		service_remain_after_exit     => $service_remain_after_exit,
		service_guess_main_pid        => $service_guess_main_pid,
		service_pid_file              => $service_pid_file,
		service_bus_name              => $service_bus_name,
		service_bus_policy            => $service_bus_policy,
		service_exec_start_pre        => $_service_exec_start_pre,
		service_exec_start            => ["/usr/bin/docker logs -f ${title}"],
		service_exec_start_post       => $service_exec_start_post,
		service_exec_stop             => ["-/usr/bin/docker stop -t 61 ${title}"],
		service_exec_stop_post        => ["-/usr/bin/docker stop -t 61 ${title}"],
		service_restart_sec           => $service_restart_sec,
		# docker pull might take a while
		service_timeout_start_sec     => 360,
		service_timeout_stop_sec      => 300,
		service_timeout_sec           => $service_timeout_sec,
		service_watchdog_sec          => $service_watchdog_sec,
		service_restart               => $service_restart,
		service_success_exit_status   => $service_success_exit_status,
		service_restart_prevnt_exit_s => $service_restart_prevnt_exit_s,
		service_force_exit_status     => $service_force_exit_status,
		service_root_dir_start_only   => $service_root_dir_start_only,
		service_non_blocking          => $service_non_blocking,
		service_notify_access         => $service_notify_access,
		service_sockets               => $service_sockets,
		service_start_limit_interval  => $service_start_limit_interval,
		service_start_limit_burst     => $service_start_limit_burst,
		service_start_limit_action    => $service_start_limit_action,
		service_failure_action        => $service_failure_action,
		service_reboot_argument       => $service_reboot_argument,
		service_fd_store_max          => $service_fd_store_max,
		# systemd.exec:
		service_working_directory     => $service_working_directory,
		service_root_directory        => $service_root_directory,
		service_user                  => $service_user,
		service_group                 => $service_group,
		service_supplementary_groups  => $service_supplementary_groups,
		service_nice                  => $service_nice,
		service_oom_score_adjust      => $service_oom_score_adjust,
		service_io_scheduling_class   => $service_io_scheduling_class,
		service_io_scheduling_prio    => $service_io_scheduling_prio,
		service_cpu_sched_policy      => $service_cpu_sched_policy,
		service_cpu_sched_priority    => $service_cpu_sched_priority,
		service_cpu_sched_rset_on_frk => $service_cpu_sched_rset_on_frk,
		service_cpu_affinity          => $service_cpu_affinity,
		service_umask                 => $service_umask,
		service_environment           => $service_environment,
		service_environment_file      => $service_environment_file,
		service_standard_input        => $service_standard_input,
		service_standard_output       => $service_standard_output,
		service_standard_error        => $service_standard_error,
		service_tty_path              => $service_tty_path,
		service_tty_reset             => $service_tty_reset,
		service_ttyv_hangup           => $service_ttyv_hangup,
		service_ttyvt_disallocate     => $service_ttyvt_disallocate,
		service_syslog_identifier     => $service_syslog_identifier,
		service_syslog_facility       => $service_syslog_facility,
		service_syslog_level          => $service_syslog_level,
		service_syslog_level_prefix   => $service_syslog_level_prefix,
		service_timer_slack_nsec      => $service_timer_slack_nsec,
		service_limit_cpu             => $service_limit_cpu,
		service_limit_fsize           => $service_limit_fsize,
		service_limit_data            => $service_limit_data,
		service_limit_stack           => $service_limit_stack,
		service_limit_core            => $service_limit_core,
		service_limit_rss             => $service_limit_rss,
		service_limit_nofile          => $service_limit_nofile,
		service_limit_as              => $service_limit_as,
		service_limit_nproc           => $service_limit_nproc,
		service_limit_memlock         => $service_limit_memlock,
		service_limit_locks           => $service_limit_locks,
		service_limit_sigpending      => $service_limit_sigpending,
		service_limit_msgqueue        => $service_limit_msgqueue,
		service_limit_nice            => $service_limit_nice,
		service_limit_rtprio          => $service_limit_rtprio,
		service_limit_rttime          => $service_limit_rttime,
		service_pam_name              => $service_pam_name,
		service_capability_boundn_set => $service_capability_boundn_set,
		service_secure_bits           => $service_secure_bits,
		service_capabilities          => $service_capabilities,
		service_read_write_dirs       => $service_read_write_dirs,
		service_read_only_dirs        => $service_read_only_dirs,
		service_inaccessible_dirs     => $service_inaccessible_dirs,
		service_private_tmp           => $service_private_tmp,
		service_private_devices       => $service_private_devices,
		service_private_network       => $service_private_network,
		service_protect_system        => $service_protect_system,
		service_protect_home          => $service_protect_home,
		service_mount_flags           => $service_mount_flags,
		service_utmp_identifier       => $service_utmp_identifier,
		service_utmp_mode             => $service_utmp_mode,
		service_se_linux_context      => $service_se_linux_context,
		service_app_armor_profile     => $service_app_armor_profile,
		service_smack_process_label   => $service_smack_process_label,
		service_ignore_sigpipe        => $service_ignore_sigpipe,
		service_no_new_privileges     => $service_no_new_privileges,
		service_system_call_filter    => $service_system_call_filter,
		service_system_call_error_no  => $service_system_call_error_no,
		service_system_call_archs     => $service_system_call_archs,
		service_restrict_address_fam  => $service_restrict_address_fam,
		service_personality           => $service_personality,
		service_runtime_dir           => $service_runtime_dir,
		service_runtime_dir_mode      => $service_runtime_dir_mode,
		# systemd.kill:
		service_kill_mode             => $service_kill_mode,
		service_kill_signal           => $service_kill_signal,
		service_send_sighup           => $service_send_sighup,
		service_send_sigkill          => $service_send_sigkill,
		# settings imported from systemd.resource-control
		service_cpu_accounting        => $service_cpu_accounting,
		service_cpu_shares            => $service_cpu_shares,
		service_startup_cpu_shares    => $service_startup_cpu_shares,
		service_cpu_quota             => $service_cpu_quota,
		service_memory_accounting     => $service_memory_accounting,
		service_memory_limit          => $service_memory_limit,
		service_blk_io_accounting     => $service_blk_io_accounting,
		service_blk_io_weight         => $service_blk_io_weight,
		service_startup_blk_io_weight => $service_startup_blk_io_weight,
		service_blk_io_device_weight  => $service_blk_io_device_weight,
		service_blk_io_read_bw        => $service_blk_io_read_bw,
		service_blk_io_write_bw       => $service_blk_io_write_bw,
		service_device_allow          => $service_device_allow,
		service_device_policy         => $service_device_policy,
		service_slice                 => $service_slice,
		service_delegate              => $service_delegate,
		# P U P P E T   E N H A N C E M E N T S
		manage_service_resource       => $manage_service_resource,
		ensure_service                => $ensure_service,
		enable_service                => $enable_service,
		hasrestart                    => $hasrestart,
		hasstatus                     => $hasstatus,
		explicit_provider             => $explicit_provider,
		refresh_service_unit_update   => $refresh_service_unit_update,
	}
}
# EOF
